@@ -1,7 +1,7 @@
-'use strict';
-
-var encoding = require('encoding');
-var sharedFuncs = require('./shared');
+const { Buffer } = require('safe-buffer');
+const encoding = require('encoding');
+const sharedFuncs = require('./shared');
+const contentType = require('content-type');
 
 /**
  * Exposes general compiler function. Takes a translation
@@ -11,7 +11,8 @@ var sharedFuncs = require('./shared');
  * @return {Buffer} Compiled binary MO object
  */
 module.exports = function (table) {
-  var compiler = new Compiler(table);
+  const compiler = new Compiler(table);
+
   return compiler.compile();
 };
 
@@ -21,10 +22,48 @@ module.exports = function (table) {
  * @constructor
  * @param {Object} table Translation table as defined in the README
  */
-function Compiler (table) {
-  this._table = table || {};
-  this._table.headers = this._table.headers || {};
-  this._table.translations = this._table.translations || {};
+function Compiler (table = {}) {
+  this._table = table;
+
+  let { headers = {}, translations = {} } = this._table;
+
+  headers = Object.keys(headers).reduce((result, key) => {
+    const lowerKey = key.toLowerCase();
+
+    if (sharedFuncs.HEADERS.has(lowerKey)) {
+      // POT-Creation-Date is removed in MO (see https://savannah.gnu.org/bugs/?49654)
+      if (lowerKey !== 'pot-creation-date') {
+        result[sharedFuncs.HEADERS.get(lowerKey)] = headers[key];
+      }
+    } else {
+      result[key] = headers[key];
+    }
+
+    return result;
+  }, {});
+
+  // filter out empty translations
+  translations = Object.keys(translations).reduce((result, msgctxt) => {
+    const context = translations[msgctxt];
+    const msgs = Object.keys(context).reduce((result, msgid) => {
+      const hasTranslation = context[msgid].msgstr.some(item => !!item.length);
+
+      if (hasTranslation) {
+        result[msgid] = context[msgid];
+      }
+
+      return result;
+    }, {});
+
+    if (Object.keys(msgs).length) {
+      result[msgctxt] = msgs;
+    }
+
+    return result;
+  }, {});
+
+  this._table.translations = translations;
+  this._table.headers = headers;
 
   this._translations = [];
 
@@ -42,35 +81,17 @@ Compiler.prototype.MAGIC = 0x950412de;
  * Handles header values, replaces or adds (if needed) a charset property
  */
 Compiler.prototype._handleCharset = function () {
-  var parts = (this._table.headers['content-type'] || 'text/plain').split(';');
-  var contentType = parts.shift();
-  var charset = sharedFuncs.formatCharset(this._table.charset);
-  var params = [];
+  const ct = contentType.parse(this._table.headers['Content-Type'] || 'text/plain');
 
-  params = parts.map(function (part) {
-    var parts = part.split('=');
-    var key = parts.shift().trim();
-    var value = parts.join('=');
+  const charset = sharedFuncs.formatCharset(this._table.charset || ct.parameters.charset || 'utf-8');
 
-    if (key.toLowerCase() === 'charset') {
-      if (!charset) {
-        charset = sharedFuncs.formatCharset(value.trim() || 'utf-8');
-      }
-      return 'charset=' + charset;
-    }
-
-    return part;
-  });
-
-  if (!charset) {
-    charset = this._table.charset || 'utf-8';
-    params.push('charset=' + charset);
+  // clean up content-type charset independently using fallback if missing
+  if (ct.parameters.charset) {
+    ct.parameters.charset = sharedFuncs.formatCharset(ct.parameters.charset);
   }
 
   this._table.charset = charset;
-  this._table.headers['content-type'] = contentType + '; ' + params.join('; ');
-
-  this._charset = charset;
+  this._table.headers['Content-Type'] = contentType.format(ct);
 };
 
 /**
@@ -80,28 +101,29 @@ Compiler.prototype._handleCharset = function () {
  * @return {Array} Translation strings array
  */
 Compiler.prototype._generateList = function () {
-  var list = [];
+  const list = [];
 
   list.push({
-    msgid: new Buffer(0),
-    msgstr: encoding.convert(sharedFuncs.generateHeader(this._table.headers), this._charset)
+    msgid: Buffer.alloc(0),
+    msgstr: encoding.convert(sharedFuncs.generateHeader(this._table.headers), this._table.charset)
   });
 
-  Object.keys(this._table.translations).forEach(function (msgctxt) {
+  Object.keys(this._table.translations).forEach(msgctxt => {
     if (typeof this._table.translations[msgctxt] !== 'object') {
       return;
     }
-    Object.keys(this._table.translations[msgctxt]).forEach(function (msgid) {
+
+    Object.keys(this._table.translations[msgctxt]).forEach(msgid => {
       if (typeof this._table.translations[msgctxt][msgid] !== 'object') {
         return;
       }
+
       if (msgctxt === '' && msgid === '') {
         return;
       }
 
-      var msgidPlural = this._table.translations[msgctxt][msgid].msgid_plural;
-      var key = msgid;
-      var value;
+      const msgidPlural = this._table.translations[msgctxt][msgid].msgid_plural;
+      let key = msgid;
 
       if (msgctxt) {
         key = msgctxt + '\u0004' + key;
@@ -111,14 +133,14 @@ Compiler.prototype._generateList = function () {
         key += '\u0000' + msgidPlural;
       }
 
-      value = [].concat(this._table.translations[msgctxt][msgid].msgstr || []).join('\u0000');
+      const value = [].concat(this._table.translations[msgctxt][msgid].msgstr || []).join('\u0000');
 
       list.push({
-        msgid: encoding.convert(key, this._charset),
-        msgstr: encoding.convert(value, this._charset)
+        msgid: encoding.convert(key, this._table.charset),
+        msgstr: encoding.convert(value, this._table.charset)
       });
-    }.bind(this));
-  }.bind(this));
+    });
+  });
 
   return list;
 };
@@ -130,11 +152,11 @@ Compiler.prototype._generateList = function () {
  * @return {Object} Size data of {msgid, msgstr, total}
  */
 Compiler.prototype._calculateSize = function (list) {
-  var msgidLength = 0;
-  var msgstrLength = 0;
-  var totalLength = 0;
+  let msgidLength = 0;
+  let msgstrLength = 0;
+  let totalLength = 0;
 
-  list.forEach(function (translation) {
+  list.forEach(translation => {
     msgidLength += translation.msgid.length + 1; // + extra 0x00
     msgstrLength += translation.msgstr.length + 1; // + extra 0x00
   });
@@ -166,33 +188,33 @@ Compiler.prototype._calculateSize = function (list) {
  * @return {Buffer} Compiled MO object
  */
 Compiler.prototype._build = function (list, size) {
-  var returnBuffer = new Buffer(size.total);
-  var curPosition = 0;
-  var i;
-  var len;
+  const returnBuffer = Buffer.alloc(size.total);
+  let curPosition = 0;
+  let i;
+  let len;
 
-    // magic
+  // magic
   returnBuffer[this._writeFunc](this.MAGIC, 0);
 
-    // revision
+  // revision
   returnBuffer[this._writeFunc](0, 4);
 
-    // string count
+  // string count
   returnBuffer[this._writeFunc](list.length, 8);
 
-    // original string table offset
+  // original string table offset
   returnBuffer[this._writeFunc](28, 12);
 
-    // translation string table offset
+  // translation string table offset
   returnBuffer[this._writeFunc](28 + (4 + 4) * list.length, 16);
 
-    // hash table size
+  // hash table size
   returnBuffer[this._writeFunc](0, 20);
 
-    // hash table offset
+  // hash table offset
   returnBuffer[this._writeFunc](28 + (4 + 4) * list.length * 2, 24);
 
-    // build originals table
+  // build originals table
   curPosition = 28 + 2 * (4 + 4) * list.length;
   for (i = 0, len = list.length; i < len; i++) {
     list[i].msgid.copy(returnBuffer, curPosition);
@@ -202,7 +224,7 @@ Compiler.prototype._build = function (list, size) {
     curPosition += list[i].msgid.length + 1;
   }
 
-    // build translations table
+  // build translations table
   for (i = 0, len = list.length; i < len; i++) {
     list[i].msgstr.copy(returnBuffer, curPosition);
     returnBuffer[this._writeFunc](list[i].msgstr.length, 28 + (4 + 4) * list.length + i * 8);
@@ -220,19 +242,10 @@ Compiler.prototype._build = function (list, size) {
  * @return {Buffer} Compiled MO object
  */
 Compiler.prototype.compile = function () {
-  var list = this._generateList();
-  var size = this._calculateSize(list);
+  const list = this._generateList();
+  const size = this._calculateSize(list);
 
-    // sort by msgid
-  list.sort(function (a, b) {
-    if (a.msgid > b.msgid) {
-      return 1;
-    }
-    if (a.msgid < b.msgid) {
-      return -1;
-    }
-    return 0;
-  });
+  list.sort(sharedFuncs.compareMsgid);
 
   return this._build(list, size);
 };
