@@ -1,7 +1,5 @@
-'use strict';
-
-var encoding = require('encoding');
-var sharedFuncs = require('./shared');
+const encoding = require('encoding');
+const sharedFuncs = require('./shared');
 
 /**
  * Parses a binary MO object into translation table
@@ -11,7 +9,8 @@ var sharedFuncs = require('./shared');
  * @return {Object} Translation object
  */
 module.exports = function (buffer, defaultCharset) {
-  var parser = new Parser(buffer, defaultCharset);
+  const parser = new Parser(buffer, defaultCharset);
+
   return parser.parse();
 };
 
@@ -22,20 +21,20 @@ module.exports = function (buffer, defaultCharset) {
  * @param {Buffer} fileContents Binary MO object
  * @param {String} [defaultCharset] Default charset to use
  */
-function Parser (fileContents, defaultCharset) {
+function Parser (fileContents, defaultCharset = 'iso-8859-1') {
   this._fileContents = fileContents;
 
-    /**
+  /**
      * Method name for writing int32 values, default littleendian
      */
   this._writeFunc = 'writeUInt32LE';
 
-    /**
+  /**
      * Method name for reading int32 values, default littleendian
      */
   this._readFunc = 'readUInt32LE';
 
-  this._charset = defaultCharset || 'iso-8859-1';
+  this._charset = defaultCharset;
 
   this._table = {
     charset: this._charset,
@@ -58,14 +57,16 @@ Parser.prototype._checkMagick = function () {
   if (this._fileContents.readUInt32LE(0) === this.MAGIC) {
     this._readFunc = 'readUInt32LE';
     this._writeFunc = 'writeUInt32LE';
+
     return true;
   } else if (this._fileContents.readUInt32BE(0) === this.MAGIC) {
     this._readFunc = 'readUInt32BE';
     this._writeFunc = 'writeUInt32BE';
+
     return true;
-  } else {
-    return false;
   }
+
+  return false;
 };
 
 /**
@@ -73,22 +74,22 @@ Parser.prototype._checkMagick = function () {
  * first translation string in the file as the header.
  */
 Parser.prototype._loadTranslationTable = function () {
-  var offsetOriginals = this._offsetOriginals;
-  var offsetTranslations = this._offsetTranslations;
-  var position;
-  var length;
-  var msgid;
-  var msgstr;
+  let offsetOriginals = this._offsetOriginals;
+  let offsetTranslations = this._offsetTranslations;
+  let position;
+  let length;
+  let msgid;
+  let msgstr;
 
-  for (var i = 0; i < this._total; i++) {
-        // msgid string
+  for (let i = 0; i < this._total; i++) {
+    // msgid string
     length = this._fileContents[this._readFunc](offsetOriginals);
     offsetOriginals += 4;
     position = this._fileContents[this._readFunc](offsetOriginals);
     offsetOriginals += 4;
     msgid = this._fileContents.slice(position, position + length);
 
-        // matching msgstr
+    // matching msgstr
     length = this._fileContents[this._readFunc](offsetTranslations);
     offsetTranslations += 4;
     position = this._fileContents[this._readFunc](offsetTranslations);
@@ -99,13 +100,15 @@ Parser.prototype._loadTranslationTable = function () {
       this._handleCharset(msgstr);
     }
 
-    msgid = encoding.convert(msgid, 'utf-8', this._charset).toString('utf-8');
-    msgstr = encoding.convert(msgstr, 'utf-8', this._charset).toString('utf-8');
+    msgid = encoding.convert(msgid, 'utf-8', this._charset)
+      .toString('utf8');
+    msgstr = encoding.convert(msgstr, 'utf-8', this._charset)
+      .toString('utf8');
 
     this._addString(msgid, msgstr);
   }
 
-    // dump the file contents object
+  // dump the file contents object
   this._fileContents = null;
 };
 
@@ -115,14 +118,15 @@ Parser.prototype._loadTranslationTable = function () {
  * @param {Buffer} headers Header value
  */
 Parser.prototype._handleCharset = function (headers) {
-  var headersStr = headers.toString();
-  var match;
+  const headersStr = headers.toString();
+  let match;
 
   if ((match = headersStr.match(/[; ]charset\s*=\s*([\w-]+)/i))) {
     this._charset = this._table.charset = sharedFuncs.formatCharset(match[1], this._charset);
   }
 
-  headers = encoding.convert(headers, 'utf-8', this._charset).toString('utf-8');
+  headers = encoding.convert(headers, 'utf-8', this._charset)
+    .toString('utf8');
 
   this._table.headers = sharedFuncs.parseHeader(headers);
 };
@@ -134,10 +138,9 @@ Parser.prototype._handleCharset = function (headers) {
  * @params {String} msgstr Translation for the original string
  */
 Parser.prototype._addString = function (msgid, msgstr) {
-  var translation = {};
-  var parts;
-  var msgctxt;
-  var msgidPlural;
+  const translation = {};
+  let msgctxt;
+  let msgidPlural;
 
   msgid = msgid.split('\u0004');
   if (msgid.length > 1) {
@@ -148,7 +151,7 @@ Parser.prototype._addString = function (msgid, msgstr) {
   }
   msgid = msgid.join('\u0004');
 
-  parts = msgid.split('\u0000');
+  const parts = msgid.split('\u0000');
   msgid = parts.shift();
 
   translation.msgid = msgid;
@@ -177,27 +180,27 @@ Parser.prototype.parse = function () {
     return false;
   }
 
-    /**
+  /**
      * GetText revision nr, usually 0
      */
   this._revision = this._fileContents[this._readFunc](4);
 
-    /**
+  /**
      * Total count of translated strings
      */
   this._total = this._fileContents[this._readFunc](8);
 
-    /**
+  /**
      * Offset position for original strings table
      */
   this._offsetOriginals = this._fileContents[this._readFunc](12);
 
-    /**
+  /**
      * Offset position for translation strings table
      */
   this._offsetTranslations = this._fileContents[this._readFunc](16);
 
-    // Load translations into this._translationTable
+  // Load translations into this._translationTable
   this._loadTranslationTable();
 
   return this._table;
