@@ -1,9 +1,7 @@
-'use strict';
-
-var encoding = require('encoding');
-var sharedFuncs = require('./shared');
-var Transform = require('stream').Transform;
-var util = require('util');
+const encoding = require('encoding');
+const sharedFuncs = require('./shared');
+const Transform = require('readable-stream').Transform;
+const util = require('util');
 
 /**
  * Parses a PO object into translation table
@@ -13,7 +11,8 @@ var util = require('util');
  * @return {Object} Translation object
  */
 module.exports.parse = function (buffer, defaultCharset) {
-  var parser = new Parser(buffer, defaultCharset);
+  const parser = new Parser(buffer, defaultCharset);
+
   return parser.parse();
 };
 
@@ -36,19 +35,20 @@ module.exports.stream = function (defaultCharset, options) {
  * @param {Buffer|String} fileContents PO object
  * @param {String} [defaultCharset] Default charset to use
  */
-function Parser (fileContents, defaultCharset) {
-  this._charset = defaultCharset || 'iso-8859-1';
+function Parser (fileContents, defaultCharset = 'iso-8859-1') {
+  this._charset = defaultCharset;
 
   this._lex = [];
   this._escaped = false;
-  this._node;
+  this._node = {};
   this._state = this.states.none;
+  this._lineNumber = 1;
 
   if (typeof fileContents === 'string') {
     this._charset = 'utf-8';
     this._fileContents = fileContents;
   } else {
-    this._handleCharset(fileContents);
+    this._fileContents = this._handleCharset(fileContents);
   }
 }
 
@@ -59,6 +59,7 @@ function Parser (fileContents, defaultCharset) {
  */
 Parser.prototype.parse = function () {
   this._lexer(this._fileContents);
+
   return this._finalize(this._lex);
 };
 
@@ -67,16 +68,15 @@ Parser.prototype.parse = function () {
  *
  * @param {Buffer} headers Header value
  */
-Parser.prototype._handleCharset = function (buf) {
-  var str = (buf || '').toString();
-  var pos;
-  var headers = '';
-  var match;
+Parser.prototype._handleCharset = function (buf = '') {
+  const str = buf.toString();
+  let pos;
+  let headers = '';
+  let match;
 
   if ((pos = str.search(/^\s*msgid/im)) >= 0) {
-    if ((pos = pos + str.substr(pos + 5).search(/^\s*(msgid|msgctxt)/im))) {
-      headers = str.substr(0, pos);
-    }
+    pos = pos + str.substr(pos + 5).search(/^\s*(msgid|msgctxt)/im);
+    headers = str.substr(0, pos >= 0 ? pos + 5 : str.length);
   }
 
   if ((match = headers.match(/[; ]charset\s*=\s*([\w-]+)(?:[\s;]|\\n)*"\s*$/mi))) {
@@ -84,10 +84,10 @@ Parser.prototype._handleCharset = function (buf) {
   }
 
   if (this._charset === 'utf-8') {
-    this._fileContents = str;
-  } else {
-    this._fileContents = this._toString(buf);
+    return str;
   }
+
+  return this._toString(buf);
 };
 
 Parser.prototype._toString = function (buf) {
@@ -120,7 +120,8 @@ Parser.prototype.symbols = {
   quotes: /["']/,
   comments: /#/,
   whitespace: /\s/,
-  key: /[\w\-[\]]/
+  key: /[\w\-[\]]/,
+  keyNames: /^(?:msgctxt|msgid(?:_plural)?|msgstr(?:\[\d+])?)$/
 };
 
 /**
@@ -129,10 +130,15 @@ Parser.prototype.symbols = {
  * @param {String} chunk String
  */
 Parser.prototype._lexer = function (chunk) {
-  var chr;
+  let chr;
 
-  for (var i = 0, len = chunk.length; i < len; i++) {
+  for (let i = 0, len = chunk.length; i < len; i++) {
     chr = chunk.charAt(i);
+
+    if (chr === '\n') {
+      this._lineNumber += 1;
+    }
+
     switch (this._state) {
       case this.states.none:
         if (chr.match(this.symbols.quotes)) {
@@ -196,6 +202,13 @@ Parser.prototype._lexer = function (chunk) {
         break;
       case this.states.key:
         if (!chr.match(this.symbols.key)) {
+          if (!this._node.value.match(this.symbols.keyNames)) {
+            const err = new SyntaxError(`Error parsing PO data: Invalid key name "${this._node.value}" at line ${this._lineNumber}. This can be caused by an unescaped quote character in a msgid or msgstr value.`);
+
+            err.lineNumber = this._lineNumber;
+
+            throw err;
+          }
           this._state = this.states.none;
           i--;
         } else {
@@ -213,10 +226,10 @@ Parser.prototype._lexer = function (chunk) {
  * @return {Object} Parsed tokens, with multi line strings joined into one
  */
 Parser.prototype._joinStringValues = function (tokens) {
-  var lastNode;
-  var response = [];
+  const response = [];
+  let lastNode;
 
-  for (var i = 0, len = tokens.length; i < len; i++) {
+  for (let i = 0, len = tokens.length; i < len; i++) {
     if (lastNode && tokens[i].type === this.types.string && lastNode.type === this.types.string) {
       lastNode.value += tokens[i].value;
     } else if (lastNode && tokens[i].type === this.types.comments && lastNode.type === this.types.comments) {
@@ -236,9 +249,10 @@ Parser.prototype._joinStringValues = function (tokens) {
  * @param {Object} tokens Parsed tokens
  */
 Parser.prototype._parseComments = function (tokens) {
-    // parse comments
-  tokens.forEach(function (node) {
-    var comment, lines;
+  // parse comments
+  tokens.forEach(node => {
+    let comment;
+    let lines;
 
     if (node && node.type === this.types.comments) {
       comment = {
@@ -248,8 +262,10 @@ Parser.prototype._parseComments = function (tokens) {
         flag: [],
         previous: []
       };
+
       lines = (node.value || '').split(/\n/);
-      lines.forEach(function (line) {
+
+      lines.forEach(line => {
         switch (line.charAt(0) || '') {
           case ':':
             comment.reference.push(line.substr(1).trim());
@@ -270,13 +286,13 @@ Parser.prototype._parseComments = function (tokens) {
 
       node.value = {};
 
-      Object.keys(comment).forEach(function (key) {
+      Object.keys(comment).forEach(key => {
         if (comment[key] && comment[key].length) {
           node.value[key] = comment[key].join('\n');
         }
       });
     }
-  }.bind(this));
+  });
 };
 
 /**
@@ -286,10 +302,10 @@ Parser.prototype._parseComments = function (tokens) {
  * @return {Object} Tokens
  */
 Parser.prototype._handleKeys = function (tokens) {
-  var response = [];
-  var lastNode;
+  const response = [];
+  let lastNode;
 
-  for (var i = 0, len = tokens.length; i < len; i++) {
+  for (let i = 0, len = tokens.length; i < len; i++) {
     if (tokens[i].type === this.types.key) {
       lastNode = {
         key: tokens[i].value
@@ -314,12 +330,12 @@ Parser.prototype._handleKeys = function (tokens) {
  * @return {Object} Tokens
  */
 Parser.prototype._handleValues = function (tokens) {
-  var response = [];
-  var lastNode;
-  var curContext;
-  var curComments;
+  const response = [];
+  let lastNode;
+  let curContext;
+  let curComments;
 
-  for (var i = 0, len = tokens.length; i < len; i++) {
+  for (let i = 0, len = tokens.length; i < len; i++) {
     if (tokens[i].key.toLowerCase() === 'msgctxt') {
       curContext = tokens[i].value;
       curComments = tokens[i].comments;
@@ -378,14 +394,14 @@ Parser.prototype._handleValues = function (tokens) {
  * @return {Object} Translation table
  */
 Parser.prototype._normalize = function (tokens) {
-  var msgctxt;
-  var table = {
+  const table = {
     charset: this._charset,
     headers: undefined,
     translations: {}
   };
+  let msgctxt;
 
-  for (var i = 0, len = tokens.length; i < len; i++) {
+  for (let i = 0, len = tokens.length; i < len; i++) {
     msgctxt = tokens[i].msgctxt || '';
 
     if (!table.translations[msgctxt]) {
@@ -409,8 +425,10 @@ Parser.prototype._normalize = function (tokens) {
  * @returns {Object} Translation table
  */
 Parser.prototype._finalize = function (tokens) {
-  var data = this._joinStringValues(tokens);
+  let data = this._joinStringValues(tokens);
+
   this._parseComments(data);
+
   data = this._handleKeys(data);
   data = this._handleValues(data);
 
@@ -449,8 +467,8 @@ util.inherits(PoParserTransform, Transform);
  * Processes a chunk of the input stream
  */
 PoParserTransform.prototype._transform = function (chunk, encoding, done) {
-  var i;
-  var len = 0;
+  let i;
+  let len = 0;
 
   if (!chunk || !chunk.length) {
     return done();
@@ -460,7 +478,7 @@ PoParserTransform.prototype._transform = function (chunk, encoding, done) {
     this._cache.push(chunk);
     this._cacheSize += chunk.length;
 
-        // wait until the first 1kb before parsing headers for charset
+    // wait until the first 1kb before parsing headers for charset
     if (this._cacheSize < this.initialTreshold) {
       return setImmediate(done);
     } else if (this._cacheSize) {
@@ -471,7 +489,7 @@ PoParserTransform.prototype._transform = function (chunk, encoding, done) {
 
     this._parser = new Parser(chunk, this.defaultCharset);
   } else if (this._cacheSize) {
-        // this only happens if we had an uncompleted 8bit sequence from the last iteration
+    // this only happens if we had an uncompleted 8bit sequence from the last iteration
     this._cache.push(chunk);
     this._cacheSize += chunk.length;
     chunk = Buffer.concat(this._cache, this._cacheSize);
@@ -479,8 +497,8 @@ PoParserTransform.prototype._transform = function (chunk, encoding, done) {
     this._cache = [];
   }
 
-    // cache 8bit bytes from the end of the chunk
-    // helps if the chunk ends in the middle of an utf-8 sequence
+  // cache 8bit bytes from the end of the chunk
+  // helps if the chunk ends in the middle of an utf-8 sequence
   for (i = chunk.length - 1; i >= 0; i--) {
     if (chunk[i] >= 0x80) {
       len++;
@@ -488,16 +506,24 @@ PoParserTransform.prototype._transform = function (chunk, encoding, done) {
     }
     break;
   }
-    // it seems we found some 8bit bytes from the end of the string, so let's cache these
+  // it seems we found some 8bit bytes from the end of the string, so let's cache these
   if (len) {
     this._cache = [chunk.slice(chunk.length - len)];
     this._cacheSize = this._cache[0].length;
     chunk = chunk.slice(0, chunk.length - len);
   }
 
-    // chunk might be empty if it only contined of 8bit bytes and these were all cached
+  // chunk might be empty if it only continued of 8bit bytes and these were all cached
   if (chunk.length) {
-    this._parser._lexer(this._parser._toString(chunk));
+    try {
+      this._parser._lexer(this._parser._toString(chunk));
+    } catch (error) {
+      setImmediate(() => {
+        done(error);
+      });
+
+      return;
+    }
   }
 
   setImmediate(done);
@@ -507,7 +533,7 @@ PoParserTransform.prototype._transform = function (chunk, encoding, done) {
  * Once all input has been processed emit the parsed translation table as an object
  */
 PoParserTransform.prototype._flush = function (done) {
-  var chunk;
+  let chunk;
 
   if (this._cacheSize) {
     chunk = Buffer.concat(this._cache, this._cacheSize);
@@ -518,7 +544,15 @@ PoParserTransform.prototype._flush = function (done) {
   }
 
   if (chunk) {
-    this._parser._lexer(this._parser._toString(chunk));
+    try {
+      this._parser._lexer(this._parser._toString(chunk));
+    } catch (error) {
+      setImmediate(() => {
+        done(error);
+      });
+
+      return;
+    }
   }
 
   if (this._parser) {
